library(igraph)
library(scales)

shinyServer(
  function(input, output) {

    # read and cache the graph
    inputGraph = reactive({
      inFile = input$graph
      if (!is.null(inFile))
        read.graph(file=inFile$datapath, format=input$format)
    })

    # read and cache the layout
    inputCoords = reactive({layout.fruchterman.reingold(inputGraph())});
    
    # render the plot
    output$net = renderPlot({
      # viualize button
      input$plot
      # isolate input graph
      g = isolate(inputGraph())
      
      if (!is.null(g)) {      
        # isolate layout
        coords = isolate(inputCoords())
        # isolate centrality measure
        cent = isolate(switch(input$centrality, 
                      "degree" = degree(g, mode = input$degree),
                      "eigenvector" = evcent(g)$vector,
                      "Katz" = alpha.centrality(g, alpha = input$alpha),
                      "pagerank" = page.rank(g, damping = input$alpha)$vector,
                      "HITS" = if (input$HITS == "authority") authority.score(g)$vector else hub.score(g)$vector,
                      "closeness" = closeness(g, mode="out"),
                      "betweenness" = betweenness(g)))
        
        # top player
        top = which.max(cent)
        V(g)$color = "skyblue"
        V(g)[top]$color = "red"
      
        plot(g, layout=coords, vertex.label=NA, vertex.size = rescale(cent, to = c(1, 10)))
      }      
    })
  }
)